@@ -1,5 +1,3 @@
-#!/usr/bin/python
-#
 # Copyright 2007 Google Inc.
 #
 # This program is free software; you can redistribute it and/or
@@ -20,6 +18,8 @@
 
 __author__ = 'vasilios@google.com (Vasilios Hoffman)'
 
+
+import __builtin__
 import errno
 import fcntl
 import os
@@ -220,6 +220,26 @@ class TestPidFile(mox.MoxTestBase):
     os.remove(proc_filename)
     os.rmdir(proc_dir)
 
+  def testSendTermNoPid(self):
+    locker = lock.PidFile()
+    self.mox.StubOutWithMock(locker, '_file')
+    locker._file.read().AndReturn('\n')
+    locker.PROC = self.workdir
+
+    self.mox.ReplayAll()
+
+    locker.SendTerm()
+
+  def testSendTermNonePid(self):
+    locker = lock.PidFile()
+    self.mox.StubOutWithMock(locker, '_file')
+    locker._file.read().AndReturn(None)
+    locker.PROC = self.workdir
+
+    self.mox.ReplayAll()
+
+    locker.SendTerm()
+
   def testSendTermTrapsENOENT(self):
     locker = lock.PidFile()
     self.mox.StubOutWithMock(locker, '_file')
@@ -227,8 +247,8 @@ class TestPidFile(mox.MoxTestBase):
     locker._file.seek(0)
     locker.PROC = self.workdir
 
-    self.mox.StubOutWithMock(__builtins__, 'open')
-    __builtins__.open(mox.IgnoreArg(), 'r').AndRaise(IOError(errno.ENOENT, ''))
+    self.mox.StubOutWithMock(__builtin__, 'open')
+    __builtin__.open(mox.IgnoreArg(), 'r').AndRaise(IOError(errno.ENOENT, ''))
 
     self.mox.ReplayAll()
 
